==> torque-2.1.9.tar.gz: torque-2.1.9/src/scheduler.cc/samples/cray_t3e/evaluate_system.c <==
/*
*         OpenPBS (Portable Batch System) v2.3 Software License
* 
* Copyright (c) 1999-2000 Veridian Information Solutions, Inc.
* All rights reserved.
* 
* ---------------------------------------------------------------------------
* For a license to use or redistribute the OpenPBS software under conditions
* other than those described below, or to purchase support for this software,
* please contact Veridian Systems, PBS Products Department ("Licensor") at:
* 
*    www.OpenPBS.org  +1 650 967-4675                  sales@OpenPBS.org
*                        877 902-4PBS (US toll-free)
* ---------------------------------------------------------------------------
* 
* This license covers use of the OpenPBS v2.3 software (the "Software") at
* your site or location, and, for certain users, redistribution of the
* Software to other sites and locations.  Use and redistribution of
* OpenPBS v2.3 in source and binary forms, with or without modification,
* are permitted provided that all of the following conditions are met.
* After December 31, 2001, only conditions 3-6 must be met:
* 
* 1. Commercial and/or non-commercial use of the Software is permitted
*    provided a current software registration is on file at www.OpenPBS.org.
*    If use of this software contributes to a publication, product, or
*    service, proper attribution must be given; see www.OpenPBS.org/credit.html
* 
* 2. Redistribution in any form is only permitted for non-commercial,
*    non-profit purposes.  There can be no charge for the Software or any
*    software incorporating the Software.  Further, there can be no
*    expectation of revenue generated as a consequence of redistributing
*    the Software.
* 
* 3. Any Redistribution of source code must retain the above copyright notice
*    and the acknowledgment contained in paragraph 6, this list of conditions
*    and the disclaimer contained in paragraph 7.
* 
* 4. Any Redistribution in binary form must reproduce the above copyright
*    notice and the acknowledgment contained in paragraph 6, this list of
*    conditions and the disclaimer contained in paragraph 7 in the
*    documentation and/or other materials provided with the distribution.
* 
* 5. Redistributions in any form must be accompanied by information on how to
*    obtain complete source code for the OpenPBS software and any
*    modifications and/or additions to the OpenPBS software.  The source code
*    must either be included in the distribution or be available for no more
*    than the cost of distribution plus a nominal fee, and all modifications
*    and additions to the Software must be freely redistributable by any party
*    (including Licensor) without restriction.
* 
* 6. All advertising materials mentioning features or use of the Software must
*    display the following acknowledgment:
* 
*     "This product includes software developed by NASA Ames Research Center,
*     Lawrence Livermore National Laboratory, and Veridian Information 
*     Solutions, Inc.
*     Visit www.OpenPBS.org for OpenPBS software support,
*     products, and information."
* 
* 7. DISCLAIMER OF WARRANTY
* 
* THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. ANY EXPRESS
* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT
* ARE EXPRESSLY DISCLAIMED.
* 
* IN NO EVENT SHALL VERIDIAN CORPORATION, ITS AFFILIATED COMPANIES, OR THE
* U.S. GOVERNMENT OR ANY OF ITS AGENCIES BE LIABLE FOR ANY DIRECT OR INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
* OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
* 
* This license will be governed by the laws of the Commonwealth of Virginia,
* without reference to its choice of law rules.
*/
/* $Id: evaluate_system.c 12 2005-02-22 20:59:54Z dev $ */

/*
 * Perform some basic analysis of system resources to determine if 
 * jobs should be run.
 */

#include <stdio.h>
#include <stdlib.h>

/* PBS header files */
#include "pbs_error.h"
#include "pbs_ifl.h"
#include "log.h"

/*  Scheduler head files */
#include "toolkit.h"
#include "gblxvars.h"

#define DONT_START_JOB 0
#define	START_JOB 1

int
schd_evaluate_system(Resources *rsrcs, char *reason)
{
    double loadchk;

    if (rsrcs->nodes_alloc == rsrcs->nodes_total) {
	if (reason)
	    (void)sprintf(reason, "No nodes available to schedule");
	return (DONT_START_JOB);
    }

    /* Are there enough jobs running?  If not, start one. */
    if ((rsrcs->njobs < schd_MIN_JOBS) && schd_MIN_JOBS) {
	if (reason)
	    (void)sprintf(reason, "Not running enough jobs (%d/%d min)",
		rsrcs->njobs, schd_MIN_JOBS);
	return (START_JOB);
    }

    /* Are there too many jobs running?  If so, don't start one. */
    if ((rsrcs->njobs >= schd_MAX_JOBS) && schd_MAX_JOBS) {
	if (reason)
	    (void)sprintf(reason, "Maximum number of jobs running (%d/%d max)",
		rsrcs->njobs, schd_MAX_JOBS);
	return (DONT_START_JOB);
    }

    /*
     * If system time is high and we are permitting deferred jobs then 
     * change our mind and don't permit deferred jobs.  The high system 
     * time could be a short duration anomaly due to several jobs all 
     * doing I/O.  However, we should let regular jobs start if we are 
     * taking idle time.
     */
    if ((rsrcs->systime >= schd_HIGH_SYSTIME) && schd_HIGH_SYSTIME) {
	if (reason)
	    (void)sprintf(reason, "High system time (%d/%d max)",
		rsrcs->systime, schd_HIGH_SYSTIME);
	return (DONT_START_JOB);
    }

    if (reason)
	(void)sprintf(reason, "Done.  Don't need to run any jobs.");

    /*
     *  If we reached here, then we really don't need to start 
     *  another job.  If we do, then the system could become over
     *  burdened with work.
     */
    return (DONT_START_JOB);
}

==> torque-2.4.6.tar.gz: torque-2.4.6/src/scheduler.cc/samples/cray_t3e/evaluate_system.c <==
/*
*         OpenPBS (Portable Batch System) v2.3 Software License
*
* Copyright (c) 1999-2000 Veridian Information Solutions, Inc.
* All rights reserved.
*
* ---------------------------------------------------------------------------
* For a license to use or redistribute the OpenPBS software under conditions
* other than those described below, or to purchase support for this software,
* please contact Veridian Systems, PBS Products Department ("Licensor") at:
*
*    www.OpenPBS.org  +1 650 967-4675                  sales@OpenPBS.org
*                        877 902-4PBS (US toll-free)
* ---------------------------------------------------------------------------
*
* This license covers use of the OpenPBS v2.3 software (the "Software") at
* your site or location, and, for certain users, redistribution of the
* Software to other sites and locations.  Use and redistribution of
* OpenPBS v2.3 in source and binary forms, with or without modification,
* are permitted provided that all of the following conditions are met.
* After December 31, 2001, only conditions 3-6 must be met:
*
* 1. Commercial and/or non-commercial use of the Software is permitted
*    provided a current software registration is on file at www.OpenPBS.org.
*    If use of this software contributes to a publication, product, or
*    service, proper attribution must be given; see www.OpenPBS.org/credit.html
*
* 2. Redistribution in any form is only permitted for non-commercial,
*    non-profit purposes.  There can be no charge for the Software or any
*    software incorporating the Software.  Further, there can be no
*    expectation of revenue generated as a consequence of redistributing
*    the Software.
*
* 3. Any Redistribution of source code must retain the above copyright notice
*    and the acknowledgment contained in paragraph 6, this list of conditions
*    and the disclaimer contained in paragraph 7.
*
* 4. Any Redistribution in binary form must reproduce the above copyright
*    notice and the acknowledgment contained in paragraph 6, this list of
*    conditions and the disclaimer contained in paragraph 7 in the
*    documentation and/or other materials provided with the distribution.
*
* 5. Redistributions in any form must be accompanied by information on how to
*    obtain complete source code for the OpenPBS software and any
*    modifications and/or additions to the OpenPBS software.  The source code
*    must either be included in the distribution or be available for no more
*    than the cost of distribution plus a nominal fee, and all modifications
*    and additions to the Software must be freely redistributable by any party
*    (including Licensor) without restriction.
*
* 6. All advertising materials mentioning features or use of the Software must
*    display the following acknowledgment:
*
*     "This product includes software developed by NASA Ames Research Center,
*     Lawrence Livermore National Laboratory, and Veridian Information
*     Solutions, Inc.
*     Visit www.OpenPBS.org for OpenPBS software support,
*     products, and information."
*
* 7. DISCLAIMER OF WARRANTY
*
* THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. ANY EXPRESS
* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT
* ARE EXPRESSLY DISCLAIMED.
*
* IN NO EVENT SHALL VERIDIAN CORPORATION, ITS AFFILIATED COMPANIES, OR THE
* U.S. GOVERNMENT OR ANY OF ITS AGENCIES BE LIABLE FOR ANY DIRECT OR INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
* OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
* This license will be governed by the laws of the Commonwealth of Virginia,
* without reference to its choice of law rules.
*/
/* $Id: evaluate_system.c 2367 2008-09-03 17:17:34Z josh $ */

/*
 * Perform some basic analysis of system resources to determine if
 * jobs should be run.
 */

#include <stdio.h>
#include <stdlib.h>

/* PBS header files */
#include "pbs_error.h"
#include "pbs_ifl.h"
#include "log.h"

/*  Scheduler head files */
#include "toolkit.h"
#include "gblxvars.h"

#define DONT_START_JOB 0
#define START_JOB 1

int
schd_evaluate_system(Resources *rsrcs, char *reason)
  {
  double loadchk;

  if (rsrcs->nodes_alloc == rsrcs->nodes_total)
    {
    if (reason)
      (void)sprintf(reason, "No nodes available to schedule");

    return (DONT_START_JOB);
    }

  /* Are there enough jobs running?  If not, start one. */
  if ((rsrcs->njobs < schd_MIN_JOBS) && schd_MIN_JOBS)
    {
    if (reason)
      (void)sprintf(reason, "Not running enough jobs (%d/%d min)",
                    rsrcs->njobs, schd_MIN_JOBS);

    return (START_JOB);
    }

  /* Are there too many jobs running?  If so, don't start one. */
  if ((rsrcs->njobs >= schd_MAX_JOBS) && schd_MAX_JOBS)
    {
    if (reason)
      (void)sprintf(reason, "Maximum number of jobs running (%d/%d max)",
                    rsrcs->njobs, schd_MAX_JOBS);

    return (DONT_START_JOB);
    }

  /*
   * If system time is high and we are permitting deferred jobs then
   * change our mind and don't permit deferred jobs.  The high system
   * time could be a short duration anomaly due to several jobs all
   * doing I/O.  However, we should let regular jobs start if we are
   * taking idle time.
   */
  if ((rsrcs->systime >= schd_HIGH_SYSTIME) && schd_HIGH_SYSTIME)
    {
    if (reason)
      (void)sprintf(reason, "High system time (%d/%d max)",
                    rsrcs->systime, schd_HIGH_SYSTIME);

    return (DONT_START_JOB);
    }

  if (reason)
    (void)sprintf(reason, "Done.  Don't need to run any jobs.");

  /*
   *  If we reached here, then we really don't need to start
   *  another job.  If we do, then the system could become over
   *  burdened with work.
   */
  return (DONT_START_JOB);
  }
